<?php 

namespace Schaffen\SerasaRemessa\Cnab600;

use Schaffen\SerasaRemessa\Utils\Record;
use Schaffen\SerasaRemessa\Utils\RecordColumn;
use Schaffen\SerasaRemessa\Utils\Util;

class Header extends Record {
    /**
     * @var int $code Codigo do Registro, fixo 0
     */
    private $code = 0;
    
    /**
     * @var int $sequence Sequência do Cabeçalho, fixo 1
     */
    private $sequence = 1;

    /**
     * @var string Identificação do arquivo fixo “SERASA-CONVEM04” 
     */
    private $file_idf = "SERASA-CONVEM04";

    /**
     * @var string digito da filial no CNPJ da empresa
     */
    private $cnpj_branch_digit;
    
    /**
     * @var array Template para criação das colunas
     */
    protected $templateColumns = [
        [
            'name' => 'code',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 1,
            'size' => 1
        ],
        [
            'name' => 'cnpj_key',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 2,
            'size' => 9
        ],
        [
            'name' => 'move_date',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 11,
            'size' => 8
        ],
        [
            'name' => 'phone_ddd',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 19,
            'size' => 4
        ],
        [
            'name' => 'phone_number',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 23,
            'size' => 8
        ],
        [
            'name' => 'phone_ext',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 31,
            'size' => 4
        ],
        [
            'name' => 'contact',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 35,
            'size' => 70
        ],
        [
            'name' => 'file_idf',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 105,
            'size' => 15
        ],
        [
            'name' => 'shipping_sequence',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 120,
            'size' => 6
        ],
        [
            'name' => 'send_code',
            'type' => RecordColumn::TYPE_ALPHABETICAL,
            'position' => 126,
            'size' => 1
        ],
        [
            'name' => 'shipping_differential',
            'type' =>  RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 127,
            'size' => 4
        ],
        [
            'name' => 'blank_1',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 131,
            'size' => 3
        ],
        [
            'name' => 'logon',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 134,
            'size' => 8
        ],
        [
            'name' => 'blank_2',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 142,
            'size' => 392
        ],
        [
            'name' => 'errors_code',
            'type' => RecordColumn::TYPE_ALPHANUMERIC,
            'position' => 534,
            'size' => 60
        ],
        [
            'name' => 'sequence',
            'type' => RecordColumn::TYPE_NUMERIC,
            'position' => 594,
            'size' => 7
        ],
    ];

    /**
     * @param array $values Valores das colunas
     */
    public function __construct($values) {
        $this->createColumns();
        $this->validate($values);
        $this->setCnpjBranchDigit($values);
        $this->setValues($values);
    }

    /**
     * Validação (a fazer)
     */
    private function validate($values) {
        // TODO
    }

    /**
     * @param array $values define os valores das colunas
     */
    private function setValues($values) {
        $values['move_date'] = Util::dateToInt($values['move_date']);
        $values['code'] = $this->code;
        $values['sequence'] = $this->sequence;
        $values['file_idf'] = $this->file_idf;
        $this->setColumnsValues($values);
    }

    /**
     * @param array $values
     */
    public function setCnpjBranchDigit($values) {
        $this->cnpj_branch_digit = $values['cnpj_branch_digit'];
        return $this->cnpj_branch_digit;
    }

    /**
     * @return string Digito de filial do CNPJ
     */
    public function getCnpjBranchDigit() {
        return $this->cnpj_branch_digit;
    }
}